<?php

include_once __DIR__ . '/constants.php';
include_once __DIR__ . '/sqlHelper.php';

/**
 * registers/starts a test run (a transaction)
 * this will create a row in the transaction table in the db using a timestamp as primary key
 * if the primary key is already used then a retry error ($status_code_MaxParallelTestsReached) is thrown
 * @param $conn PDO the db connection
 * @param $config object the config to know the transaction table
 * @param $rootDirName int the root dir of the current iteration (this could be used to remove old transactions in the future)
 * @param $testType string the test type (just as info)
 * @param $requestDistinctionString {string} unique string (MUST BE A VALID FILE NAME) to help the this server to separate two requests in case they arrive at the same time (and micro time would return the same value)
 * @return int the timestamp used (to end/close the transaction)
 */
function startTransaction($conn, $config, $rootDirName, $testType, $requestDistinctionString)
{

    global $s_arg_dbTransactionTableName;
    global $status_code_DbIssue;
    global $s_arg_maxParallelTests;
    global $status_code_InternalServerError;
    global $status_code_MaxParallelTestsReached;

    $tableName = $config[$s_arg_dbTransactionTableName];
    $maxParallelTests = $config[$s_arg_maxParallelTests];

    if ($maxParallelTests === NULL) {
        output($status_code_InternalServerError, 'could not read maxParallelTests from config');
        exit(1);
    }


    //this gives as precision in s ... that's too bad...
    //$now = new DateTime('now');
    //date_timestamp_get($now);

    //use micro seconds...
    list($usec, $sec) = explode(" ", microtime());
    $timestamp = $sec . '_' . str_replace('.', '-', $usec) . '';

    if ($requestDistinctionString !== NULL) {
        $timestamp = $timestamp . '-' . $requestDistinctionString;
    }


    if ($maxParallelTests > 0) {
        # only check max parallel tests if ask to

        # check if the max parallel tests are running --> yes throw retry error

        $sql = "SELECT * FROM `" . $tableName . "`;";

        $currentTransactions = executeSqlAndReturnValue($conn, $sql, []);

        if (count($currentTransactions) >= $maxParallelTests) {
            //limit reached, throw

            output($status_code_MaxParallelTestsReached, 'max parallel tests reached, retry later');
            $date = new DateTime();
            warn('max parallel tests reached! : ' . $date->format('Y-m-d H:i:s'));
            exit(1);
        }
    }


    # start the transaction

    $sql = "INSERT INTO `" . $tableName . "` (id,rootDirName, testType) VALUES (:id, :rootDirName, :testType);";
    $params = array(
        ':id' => $timestamp,
        ':rootDirName' => $rootDirName,
        ':testType' => $testType
    );

    if (executeSql($conn, $sql, $params) === NULL) {
        //some error
        output($status_code_DbIssue, 'could not start transaction');
        exit(1);
    }

    return $timestamp;
}

/**
 * ends a transaction (removes the row in the db)
 *
 * @param $conn PDO the connection
 * @param $config object the config tp know the table name
 * @param $rootDirName int the root dir of the current iteration (this could be used to remove old transactions in the future)
 * @param $transactionKey int the primary key
 */
function endTransaction($conn, $config, $transactionKey, $rootDirName)
{

    global $s_arg_dbTransactionTableName;
    global $status_code_DbIssue;


    $tableName = $config[$s_arg_dbTransactionTableName];
    $sql = "DELETE FROM `" . $tableName . "` WHERE id = :id;";
    $params = array(
        ':id' => $transactionKey
    );

    if (executeSql($conn, $sql, $params) === NULL) {
        //some error
        output($status_code_DbIssue, 'could not end transaction');
        exit(1);
    }
}

/**
 * @param $conn PDO the db connection
 * @param $config
 * @param $traceString string the trace string from the request
 * @param $processString string this should be the process name or arguments to find it in process managers
 * @param $sourceCode string all user program source code for from request
 */
function addTraceLogToDb($conn, $config, $traceString, $processString, $sourceCode)
{

    global $s_arg_dbTraceTableName;
    global $status_code_DbIssue;
    global $s_arg_insertTraceLogs;


    //0 = never delete AND never insert
    //-x = never delete
    $insertTraceLogs = boolval($config[$s_arg_insertTraceLogs]);

    if ($insertTraceLogs === TRUE) {

        debug(strlen($sourceCode));

        $tableName = $config[$s_arg_dbTraceTableName];

        $sql = "INSERT INTO `" . $tableName . "` (traceString, processString, sourceCode) VALUES (:traceString, :processString, :sourceCode);";
        $params = array(
            ':traceString' => $traceString,
            ':processString' => $processString,
            ':sourceCode' => $sourceCode
        );

        if (executeSql($conn, $sql, $params) === NULL) {
            //some error
            output($status_code_DbIssue, 'could not add trace log');
            exit(1);
        }
    }

}

/**
 * @param $conn PDO
 * @param $config
 * @param $s_deleteTraceLogOlderThanMinutes string caller must check if this is a number or not
 */
function removeOldTraceLogs($conn, $config, $s_deleteTraceLogOlderThanMinutes) {

    global $s_arg_dbTraceTableName;
    global $status_code_DbIssue;

    $tableName = $config[$s_arg_dbTraceTableName];
    $deleteTraceLogOlderThanMinutes = intval($s_deleteTraceLogOlderThanMinutes); //gives 0 on error

    //$deleteTraceLogOlderThanMinutes = 0 or -x -> never delete

    //delete old entries... this has a performance impact... maybe move this to a cron job or something??
    if (is_numeric($s_deleteTraceLogOlderThanMinutes)) {

        if ($deleteTraceLogOlderThanMinutes > 0) {

            $sql = "DELETE FROM `" . $tableName . "` WHERE dateTime < (NOW() - INTERVAL :olderThan MINUTE);";
            $params = array(
                ':olderThan' => $deleteTraceLogOlderThanMinutes
            );

            if (executeSql($conn, $sql, $params) === NULL) {
                //some error
                output($status_code_DbIssue, 'could not remove old trace logs');
                exit(1);
            }
        }
        else if ($deleteTraceLogOlderThanMinutes === 0){

            $sql = "DELETE FROM `" . $tableName . "`;";
            $params = array(
                ':olderThan' => $deleteTraceLogOlderThanMinutes
            );

            if (executeSql($conn, $sql, $params) === NULL) {
                //some error
                output($status_code_DbIssue, 'could not remove all old trace logs');
                exit(1);
            }
        }

    }

}
